@@ -1,4 +1,4 @@
-// Copyright 2019 Northern.tech AS
+// Copyright 2020 Northern.tech AS
 //
 //    Licensed under the Apache License, Version 2.0 (the "License");
 //    you may not use this file except in compliance with the License.
@@ -115,6 +115,7 @@ func MakeModuleImageArtifact(signed, hasScripts bool, updateType string,
 		if err != nil {
 			return nil, err
 		}
+		defer os.Remove(file)
 		files[index] = &handlers.DataFile{Name: file}
 	}
 	compose.SetUpdateFiles(files)
@@ -128,6 +129,7 @@ func MakeModuleImageArtifact(signed, hasScripts bool, updateType string,
 			if err != nil {
 				return nil, err
 			}
+			defer os.Remove(file)
 			augmentFiles[index] = &handlers.DataFile{Name: file}
 		}
 		compose.SetUpdateAugmentFiles(augmentFiles)
@@ -389,6 +391,7 @@ type installer struct {
 	t          *testing.T
 	Data       *handlers.DataFile
 	updateType string
+	typeInfoV3 *artifact.TypeInfoV3
 }
 
 func (i *installer) GetUpdateFiles() [](*handlers.DataFile) {
@@ -415,36 +418,42 @@ func (i *installer) SetUpdateAugmentFiles(files [](*handlers.DataFile)) error {
 	return nil
 }
 
-func (i *installer) GetUpdateDepends() (*artifact.TypeInfoDepends, error) {
-	return &artifact.TypeInfoDepends{}, nil
+func (i *installer) GetUpdateDepends() (artifact.TypeInfoDepends, error) {
+	// NOTE: For testing purposes, only typeInfoV3 is returned.
+	//Original functionality merges typeInfo, and the augmentedTypeInfo,
+	// So beware!
+	return i.typeInfoV3.ArtifactDepends, nil
 }
 
-func (i *installer) GetUpdateProvides() (*artifact.TypeInfoProvides, error) {
-	return &artifact.TypeInfoProvides{}, nil
+func (i *installer) GetUpdateProvides() (artifact.TypeInfoProvides, error) {
+	// NOTE: For testing purposes, only typeInfoV3 is returned.
+	//Original functionality merges typeInfo, and the augmentedTypeInfo,
+	// So beware!
+	return i.typeInfoV3.ArtifactProvides, nil
 }
 
 func (i *installer) GetUpdateMetaData() (map[string]interface{}, error) {
 	return nil, nil
 }
 
-func (i *installer) GetUpdateOriginalDepends() *artifact.TypeInfoDepends {
-	return &artifact.TypeInfoDepends{}
+func (i *installer) GetUpdateOriginalDepends() artifact.TypeInfoDepends {
+	return artifact.TypeInfoDepends{}
 }
 
-func (i *installer) GetUpdateOriginalProvides() *artifact.TypeInfoProvides {
-	return &artifact.TypeInfoProvides{}
+func (i *installer) GetUpdateOriginalProvides() artifact.TypeInfoProvides {
+	return artifact.TypeInfoProvides{}
 }
 
 func (i *installer) GetUpdateOriginalMetaData() map[string]interface{} {
 	return nil
 }
 
-func (i *installer) GetUpdateAugmentDepends() *artifact.TypeInfoDepends {
-	return &artifact.TypeInfoDepends{}
+func (i *installer) GetUpdateAugmentDepends() artifact.TypeInfoDepends {
+	return artifact.TypeInfoDepends{}
 }
 
-func (i *installer) GetUpdateAugmentProvides() *artifact.TypeInfoProvides {
-	return &artifact.TypeInfoProvides{}
+func (i *installer) GetUpdateAugmentProvides() artifact.TypeInfoProvides {
+	return artifact.TypeInfoProvides{}
 }
 
 func (i *installer) GetUpdateAugmentMetaData() map[string]interface{} {
@@ -1304,3 +1313,222 @@ func TestReadBrokenArtifact(t *testing.T) {
 		})
 	}
 }
+
+func TestMergeDependsSuccess(t *testing.T) {
+	tests := []struct {
+		r        Reader
+		expected map[string]interface{}
+	}{
+		{
+			// No matching keys
+			r: Reader{
+				hInfo: &artifact.HeaderInfoV3{
+					ArtifactDepends: &artifact.ArtifactDepends{
+						ArtifactName:      []string{"foo"},
+						CompatibleDevices: []string{"qemux86-64"},
+						ArtifactGroup:     []string{"ac-dc"},
+					},
+				},
+				installers: map[int]handlers.Installer{
+					1: &installer{
+						typeInfoV3: &artifact.TypeInfoV3{
+							ArtifactDepends: artifact.TypeInfoDepends{
+								"foo": "boo",
+							},
+						},
+					},
+				},
+			},
+			expected: map[string]interface{}{
+				"artifact_name":      []string{"foo"},
+				"compatible_devices": []string{"qemux86-64"},
+				"artifact_group":     []string{"ac-dc"},
+				"foo":                "boo",
+			},
+		},
+		{
+			// No type-info depends
+			r: Reader{
+				hInfo: &artifact.HeaderInfoV3{
+					ArtifactDepends: &artifact.ArtifactDepends{
+						ArtifactName:      []string{"foo"},
+						CompatibleDevices: []string{"qemux86-64"},
+						ArtifactGroup:     []string{"ac-dc"},
+					},
+				},
+				installers: map[int]handlers.Installer{
+					1: &installer{
+						typeInfoV3: &artifact.TypeInfoV3{
+							ArtifactDepends: nil,
+						},
+					},
+				},
+			},
+			expected: map[string]interface{}{
+				"artifact_name":      []string{"foo"},
+				"compatible_devices": []string{"qemux86-64"},
+				"artifact_group":     []string{"ac-dc"},
+			},
+		},
+		{
+			// Artifact version 2
+			r: Reader{
+				hInfo: &artifact.HeaderInfo{
+					ArtifactName:      "foo",
+					CompatibleDevices: []string{"qemux86-64"},
+				},
+				installers: map[int]handlers.Installer{
+					1: &installer{},
+				},
+			},
+			expected: nil,
+		},
+	}
+
+	for _, test := range tests {
+		d, err := test.r.MergeArtifactDepends()
+		assert.Nil(t, err)
+		assert.Equal(t, d, test.expected)
+	}
+}
+
+func TestMergeDependsError(t *testing.T) {
+	tests := []struct {
+		r Reader
+	}{
+		{
+			// Single update -- matching keys in header-info and type-info
+			// With different types
+			r: Reader{
+				hInfo: &artifact.HeaderInfoV3{
+					ArtifactDepends: &artifact.ArtifactDepends{
+						ArtifactName:      []string{"foo"},
+						CompatibleDevices: []string{"qemux86-64"},
+						ArtifactGroup:     []string{"ac-dc"},
+					},
+				},
+				installers: map[int]handlers.Installer{
+					1: &installer{
+						typeInfoV3: &artifact.TypeInfoV3{
+							ArtifactDepends: artifact.TypeInfoDepends{
+								"artifact_name": "boo",
+							},
+						},
+					},
+				},
+			},
+		},
+	}
+	for _, test := range tests {
+		_, err := test.r.MergeArtifactDepends()
+		assert.NotNil(t, err)
+	}
+}
+
+func TestMergeProvidesSuccess(t *testing.T) {
+	tests := []struct {
+		r        Reader
+		expected map[string]interface{}
+	}{
+		{
+			// Single update -- string type-info
+			r: Reader{
+				hInfo: &artifact.HeaderInfoV3{
+					ArtifactProvides: &artifact.ArtifactProvides{
+						ArtifactName:  "foo",
+						ArtifactGroup: "ac-dc",
+					},
+				},
+				installers: map[int]handlers.Installer{
+					1: &installer{
+						typeInfoV3: &artifact.TypeInfoV3{
+							ArtifactProvides: artifact.TypeInfoProvides{
+								"bar": "baz",
+								"foo": "boo",
+							},
+						},
+					},
+				},
+			},
+			expected: map[string]interface{}{
+				"artifact_name":  "foo",
+				"artifact_group": "ac-dc",
+				"bar":            "baz",
+				"foo":            "boo",
+			},
+		},
+		{
+			// Single update -- empty type-info
+			r: Reader{
+				hInfo: &artifact.HeaderInfoV3{
+					ArtifactProvides: &artifact.ArtifactProvides{
+						ArtifactName:  "foo",
+						ArtifactGroup: "ac-dc",
+					},
+				},
+				installers: map[int]handlers.Installer{
+					1: &installer{
+						typeInfoV3: &artifact.TypeInfoV3{
+							ArtifactProvides: nil,
+						},
+					},
+				},
+			},
+			expected: map[string]interface{}{
+				"artifact_name":  "foo",
+				"artifact_group": "ac-dc",
+			},
+		},
+		{
+			// Single update -- version 2 header
+			r: Reader{
+				hInfo: &artifact.HeaderInfo{
+					ArtifactName:      "foo",
+					CompatibleDevices: []string{"ac-dc"},
+				},
+				installers: map[int]handlers.Installer{
+					1: &installer{},
+				},
+			},
+			expected: nil,
+		},
+	}
+
+	for _, test := range tests {
+		d, err := test.r.MergeArtifactProvides()
+		assert.Nil(t, err)
+		assert.Equal(t, d, test.expected)
+	}
+}
+
+func TestMergeProvidesError(t *testing.T) {
+	tests := []struct {
+		r Reader
+	}{
+		{
+			// Single update -- value is not string or []string
+			r: Reader{
+				hInfo: &artifact.HeaderInfoV3{
+					ArtifactProvides: &artifact.ArtifactProvides{
+						ArtifactName:  "foo",
+						ArtifactGroup: "ac-dc",
+					},
+				},
+				installers: map[int]handlers.Installer{
+					1: &installer{
+						typeInfoV3: &artifact.TypeInfoV3{
+							ArtifactProvides: artifact.TypeInfoProvides{
+								"artifact_name": 1,
+							},
+						},
+					},
+				},
+			},
+		},
+	}
+
+	for _, test := range tests {
+		_, err := test.r.MergeArtifactProvides()
+		assert.NotNil(t, err)
+	}
+}
