@@ -1,4 +1,4 @@
-// Copyright 2019 Northern.tech AS
+// Copyright 2020 Northern.tech AS
 //
 //    Licensed under the Apache License, Version 2.0 (the "License");
 //    you may not use this file except in compliance with the License.
@@ -335,10 +335,10 @@ func TestMarshalJSONTypeInfoV3(t *testing.T) {
 		"delta": {
 			ti: TypeInfoV3{
 				Type: "delta",
-				ArtifactDepends: &TypeInfoDepends{
+				ArtifactDepends: TypeInfoDepends{
 					"rootfs_image_checksum": "4d480539cdb23a4aee6330ff80673a5af92b7793eb1c57c4694532f96383b619",
 				},
-				ArtifactProvides: &TypeInfoProvides{
+				ArtifactProvides: TypeInfoProvides{
 					"rootfs_image_checksum": "853jsdfh342789sdflkjsdf987324kljsdf987234kjljsdf987234klsdf987d8",
 				},
 			},
