@@ -1,4 +1,4 @@
-// Copyright 2019 Northern.tech AS
+// Copyright 2020 Northern.tech AS
 //
 //    Licensed under the Apache License, Version 2.0 (the "License");
 //    you may not use this file except in compliance with the License.
@@ -39,7 +39,13 @@ func Cat(c *cli.Context) (err error) {
 	if !isimg.MatchString(c.Args().First()) {
 		return cli.NewExitError("The input image does not seem to be a valid image", 1)
 	}
-	r, err := virtualPartitionFile.Open(comp, c.Args().First())
+
+	privateKey, err := getKey(c.String("key"))
+	if err != nil {
+		return cli.NewExitError("Unable to load key: "+err.Error(), 1)
+	}
+
+	r, err := virtualImage.OpenFile(comp, privateKey, c.Args().First())
 	defer func() {
 		if r == nil {
 			return
@@ -65,6 +71,11 @@ func Copy(c *cli.Context) (err error) {
 		return cli.NewExitError("compressor '"+c.GlobalString("compression")+"' is not supported: "+err.Error(), 1)
 	}
 
+	privateKey, err := getKey(c.String("key"))
+	if err != nil {
+		return cli.NewExitError("Unable to load key: "+err.Error(), 1)
+	}
+
 	var r io.ReadCloser
 	var w io.WriteCloser
 	wclose := func(w io.Closer) {
@@ -84,7 +95,7 @@ func Copy(c *cli.Context) (err error) {
 		if err != nil {
 			return cli.NewExitError(err, 1)
 		}
-		vfile, err = virtualPartitionFile.Open(comp, c.Args().Get(1))
+		vfile, err = virtualImage.OpenFile(comp, privateKey, c.Args().Get(1))
 		defer wclose(vfile)
 		if err != nil {
 			return cli.NewExitError(err, 1)
@@ -95,14 +106,14 @@ func Copy(c *cli.Context) (err error) {
 		return nil
 	case copyinstdin:
 		r = os.Stdin
-		vfile, err = virtualPartitionFile.Open(comp, c.Args().Get(1))
+		vfile, err = virtualImage.OpenFile(comp, privateKey, c.Args().Get(1))
 		defer wclose(vfile)
 		if err != nil {
 			return cli.NewExitError(err, 1)
 		}
 		w = vfile
 	case copyout:
-		vfile, err = virtualPartitionFile.Open(comp, c.Args().First())
+		vfile, err = virtualImage.OpenFile(comp, privateKey, c.Args().First())
 		defer wclose(vfile)
 		if err != nil {
 			return cli.NewExitError(err, 1)
@@ -134,6 +145,11 @@ func Install(c *cli.Context) (err error) {
 		return cli.NewExitError("compressor '"+c.GlobalString("compression")+"' is not supported: "+err.Error(), 1)
 	}
 
+	privateKey, err := getKey(c.String("key"))
+	if err != nil {
+		return cli.NewExitError("Unable to load key: "+err.Error(), 1)
+	}
+
 	wclose := func(w io.Closer) {
 		if w == nil {
 			return
@@ -155,11 +171,6 @@ func Install(c *cli.Context) (err error) {
 		if err != nil {
 			return cli.NewExitError(fmt.Sprintf("%v", err), 1)
 		}
-		vfile, err := virtualPartitionFile.Open(comp, c.Args().Get(1))
-		if err != nil {
-			return cli.NewExitError(fmt.Sprintf("%v", err), 1)
-		}
-		defer wclose(vfile)
 
 		tfName, err := createTmpFileWithPerm(f, perm)
 		if err != nil {
@@ -174,6 +185,12 @@ func Install(c *cli.Context) (err error) {
 			}
 		}()
 
+		vfile, err := virtualImage.OpenFile(comp, privateKey, c.Args().Get(1))
+		defer wclose(vfile)
+		if err != nil {
+			return cli.NewExitError(fmt.Sprintf("%v", err), 1)
+		}
+
 		if err = vfile.CopyTo(tfName); err != nil {
 			return cli.NewExitError(err, 1)
 		}
@@ -197,6 +214,12 @@ func Remove(c *cli.Context) (err error) {
 			err = cerr
 		}
 	}
+
+	privateKey, err := getKey(c.String("key"))
+	if err != nil {
+		return cli.NewExitError("Unable to load key: "+err.Error(), 1)
+	}
+
 	comp, err := artifact.NewCompressorFromId(c.GlobalString("compression"))
 	if err != nil {
 		return cli.NewExitError("compressor '"+c.GlobalString("compression")+"' is not supported: "+err.Error(), 1)
@@ -207,7 +230,7 @@ func Remove(c *cli.Context) (err error) {
 	if !isimg.MatchString(c.Args().First()) {
 		return cli.NewExitError("The input image does not have a valid extension", 1)
 	}
-	f, err := virtualPartitionFile.Open(comp, c.Args().First())
+	f, err := virtualImage.OpenFile(comp, privateKey, c.Args().First())
 	defer wclose(f)
 	if err != nil {
 		return cli.NewExitError(fmt.Sprintf("failed to open the partition reader: err: %v", err), 1)
