@@ -1,4 +1,4 @@
-// Copyright 2019 Northern.tech AS
+// Copyright 2020 Northern.tech AS
 //
 //    Licensed under the Apache License, Version 2.0 (the "License");
 //    you may not use this file except in compliance with the License.
@@ -52,16 +52,7 @@ func applyCompressionInCommand(c *cli.Context) error {
 	// Let --compression argument work after command as well. Latest one
 	// applies.
 	if c.String("compression") != "" {
-		parent := c
-		// Find top level context, where the original --compression
-		// argument lives.
-		for {
-			if parent.Parent() == nil {
-				break
-			}
-			parent = parent.Parent()
-		}
-		parent.Set("compression", c.String("compression"))
+		c.GlobalSet("compression", c.String("compression"))
 	}
 	return nil
 }
@@ -79,7 +70,7 @@ func getCliContext() *cli.App {
 	compressors := artifact.GetRegisteredCompressorIds()
 
 	compressionFlag := cli.StringFlag{
-		Name:  "compression",
+		Name: "compression",
 		Usage: fmt.Sprintf("Compression to use for data and header inside the artifact, "+
 			"currently supports: %v.", strings.Join(compressors, ", ")),
 	}
@@ -89,40 +80,73 @@ func getCliContext() *cli.App {
 
 	privateKeyFlag := cli.StringFlag{
 		Name: "key, k",
-		Usage: "Full path to the private key that will be used to verify " +
-			"the artifact signature.",
+		Usage: "Full path to the private key that will be used to sign " +
+			"the Artifact.",
 	}
 
 	publicKeyFlag := cli.StringFlag{
 		Name: "key, k",
 		Usage: "Full path to the public key that will be used to verify " +
-			"the artifact signature.",
+			"the Artifact signature.",
+	}
+
+	//
+	// Common Artifact flags
+	//
+	artifactName := cli.StringFlag{
+		Name:  "artifact-name, n",
+		Usage: "Name of the artifact",
+	}
+	artifactNameDepends := cli.StringSliceFlag{
+		Name:  "artifact-name-depends, N",
+		Usage: "Sets the name(s) of the artifact(s) which this update depends upon",
+	}
+	artifactProvidesGroup := cli.StringFlag{
+		Name:  "provides-group, g",
+		Usage: "The group the artifact provides",
+	}
+	artifactDependsGroups := cli.StringSliceFlag{
+		Name:  "depends-groups, G",
+		Usage: "The group(s) the artifact depends on",
+	}
+
+	//
+	// Common Payload flags
+	//
+	payloadProvides := cli.StringSliceFlag{
+		Name:  "provides, p",
+		Usage: "Generic `KEY:VALUE` which is added to the type-info -> artifact_provides section. Can be given multiple times",
+	}
+	payloadDepends := cli.StringSliceFlag{
+		Name:  "depends, d",
+		Usage: "Generic `KEY:VALUE` which is added to the type-info -> artifact_depends section. Can be given multiple times",
+	}
+	payloadMetaData := cli.StringFlag{
+		Name:  "meta-data, m",
+		Usage: "The meta-data JSON `FILE` for this payload",
 	}
 
 	//
 	// write
 	//
 	writeRootfsCommand := cli.Command{
-		Name:      "rootfs-image",
-		Action:    writeRootfs,
-		Usage:     "Writes Mender artifact containing rootfs image",
-		ArgsUsage: "<image path>",
+		Name:   "rootfs-image",
+		Action: writeRootfs,
+		Usage:  "Writes Mender artifact containing rootfs image",
 	}
 
 	writeRootfsCommand.Flags = []cli.Flag{
 		cli.StringFlag{
-			Name:  "file, f",
-			Usage: "Payload `FILE`.",
+			Name: "file, f",
+			Usage: "Payload `FILE` path or ssh-url to device for system " +
+				"snapshot (e.g. ssh://user@device:22022).",
 		},
 		cli.StringSliceFlag{
 			Name: "device-type, t",
 			Usage: "Type of device(s) supported by the Artifact. You can specify multiple " +
 				"compatible devices providing this parameter multiple times.",
 		},
-		cli.StringFlag{
-			Name:  "artifact-name, n",
-			Usage: "Name of the artifact",
-		},
+		artifactName,
 		cli.StringFlag{
 			Name:  "output-path, o",
 			Usage: "Full path to output artifact file.",
@@ -138,33 +162,26 @@ func getCliContext() *cli.App {
 			Usage: "Full path to the state script(s). You can specify multiple " +
 				"scripts providing this parameter multiple times.",
 		},
+		cli.BoolFlag{
+			Name: "no-checksum-provide",
+			Usage: "Disable writing the provides checksum to the Artifact provides " +
+				"parameters. This is needed in case the targeted devices do not support " +
+				"provides and depends yet.",
+		},
+		cli.StringSliceFlag{
+			Name: "ssh-args, S",
+			Usage: "Arguments to pass to ssh - only applies when " +
+				"creating artifact from snapshot (i.e. FILE " +
+				"contains 'ssh://' schema)",
+		},
 		/////////////////////////
 		// Version 3 specifics.//
 		/////////////////////////
-		// Hiding these for now. We are not enforcing them on the client
-		// so we should not set these yet.
-		//
-		// cli.StringSliceFlag{
-		// 	Name:  "artifact-name-depends, N",
-		// 	Usage: "Sets the name(s) of the artifact(s) which this update depends upon",
-		// },
-		// cli.StringFlag{
-		// 	Name:  "provides-group, g",
-		// 	Usage: "The group the artifact provides",
-		// },
-		// cli.StringSliceFlag{
-		// 	Name:  "depends-groups, G",
-		// 	Usage: "The group(s) the artifact depends on",
-		// },
-		// cli.StringFlag{
-		// 	Name:  "depends-rootfs-image-checksum",
-		// 	Usage: "The checksum of the rootfs image which this artifact depends upon",
-		// },
-		// cli.StringFlag{
-		// 	Name:  "provides-rootfs-image-checksum",
-		// 	Usage: "The checksum of the rootfs image which this artifact provides",
-		// },
-
+		artifactNameDepends,
+		artifactProvidesGroup,
+		artifactDependsGroups,
+		payloadDepends,
+		payloadProvides,
 		compressionFlag,
 	}
 	writeRootfsCommand.Before = applyCompressionInCommand
@@ -188,10 +205,6 @@ func getCliContext() *cli.App {
 			Usage: "Type of device(s) supported by the Artifact. You can specify multiple " +
 				"compatible devices providing this parameter multiple times.",
 		},
-		cli.StringFlag{
-			Name:  "artifact-name, n",
-			Usage: "Name of the artifact",
-		},
 		cli.StringFlag{
 			Name:  "output-path, o",
 			Usage: "Full path to output artifact file.",
@@ -210,34 +223,17 @@ func getCliContext() *cli.App {
 			Usage: "Full path to the state script(s). You can specify multiple " +
 				"scripts providing this parameter multiple times.",
 		},
-		cli.StringSliceFlag{
-			Name:  "artifact-name-depends, N",
-			Usage: "Sets the name(s) of the artifact(s) which this update depends upon",
-		},
-		cli.StringFlag{
-			Name:  "provides-group, g",
-			Usage: "The group the artifact provides",
-		},
-		cli.StringSliceFlag{
-			Name:  "depends-groups, G",
-			Usage: "The group(s) the artifact depends on",
-		},
+		artifactName,
+		artifactNameDepends,
+		artifactProvidesGroup,
+		artifactDependsGroups,
 		cli.StringFlag{
 			Name:  "type, T",
 			Usage: "Type of payload. This is the same as the name of the update module",
 		},
-		cli.StringSliceFlag{
-			Name:  "provides, p",
-			Usage: "Generic `KEY:VALUE` which is added to the type-info -> artifact_provides section. Can be given multiple times",
-		},
-		cli.StringSliceFlag{
-			Name:  "depends, d",
-			Usage: "Generic `KEY:VALUE` which is added to the type-info -> artifact_depends section. Can be given multiple times",
-		},
-		cli.StringFlag{
-			Name:  "meta-data, m",
-			Usage: "The meta-data JSON `FILE` for this payload",
-		},
+		payloadProvides,
+		payloadDepends,
+		payloadMetaData,
 		cli.StringSliceFlag{
 			Name:  "file, f",
 			Usage: "Include `FILE` in payload. Can be given more than once.",
@@ -349,17 +345,30 @@ func getCliContext() *cli.App {
 			Usage: "Full path to the public verification key that is used by the client " +
 				"to verify the artifact.",
 		},
+		artifactName,
 		cli.StringFlag{
-			Name:  "name, n",
-			Usage: "New name of the artifact.",
+			Name:  "name",
+			Usage: "Deprecated. This is an alias for --artifact-name",
 		},
+		artifactNameDepends,
+		artifactProvidesGroup,
+		artifactDependsGroups,
+		payloadProvides,
+		payloadDepends,
+		payloadMetaData,
 		cli.StringFlag{
 			Name:  "tenant-token, t",
 			Usage: "Full path to the tenant token that will be injected into modified file.",
 		},
+		privateKeyFlag,
 		compressionFlag,
 	}
-	modify.Before = applyCompressionInCommand
+	modify.Before = func(c *cli.Context) error {
+		if c.String("name") != "" {
+			c.Set("artifact-name", c.String("name"))
+		}
+		return applyCompressionInCommand(c)
+	}
 
 	copy := cli.Command{
 		Name:        "cp",
@@ -373,6 +382,7 @@ func getCliContext() *cli.App {
 
 	copy.Flags = []cli.Flag{
 		compressionFlag,
+		privateKeyFlag,
 	}
 	copy.Before = applyCompressionInCommand
 
