@@ -1,4 +1,4 @@
-// Copyright 2019 Northern.tech AS
+// Copyright 2020 Northern.tech AS
 //
 //    Licensed under the Apache License, Version 2.0 (the "License");
 //    you may not use this file except in compliance with the License.
@@ -156,7 +156,7 @@ func TestWithScripts(t *testing.T) {
 	fakeErrWriter.Reset()
 	err = run()
 	assert.Error(t, err)
-	assert.Equal(t, "Error: Mender-Artifact version 1 is not supported\n",
+	assert.Equal(t, "Artifact version 1 is not supported\n",
 		fakeErrWriter.String())
 
 	// write artifact vith invalid script name
@@ -256,13 +256,13 @@ func TestWriteModuleImage(t *testing.T) {
 		"testDependKey1":    "testDependValue1",
 		"testDependKey2":    "testDependValue2",
 		"overrideDependKey": "originalOverrideDependValue",
-	}, *updDepends)
+	}, updDepends)
 	updDepends = handler.GetUpdateAugmentDepends()
 	assert.Equal(t, artifact.TypeInfoDepends{
 		"augmentDependKey1": "augmentDependValue1",
 		"augmentDependKey2": "augmentDependValue2",
 		"overrideDependKey": "augmentOverrideDependValue",
-	}, *updDepends)
+	}, updDepends)
 	updDepends, err = handler.GetUpdateDepends()
 	require.NoError(t, err)
 	assert.Equal(t, artifact.TypeInfoDepends{
@@ -271,20 +271,20 @@ func TestWriteModuleImage(t *testing.T) {
 		"augmentDependKey1": "augmentDependValue1",
 		"augmentDependKey2": "augmentDependValue2",
 		"overrideDependKey": "augmentOverrideDependValue",
-	}, *updDepends)
+	}, updDepends)
 
 	updProvides := handler.GetUpdateOriginalProvides()
 	assert.Equal(t, artifact.TypeInfoProvides{
 		"testProvideKey1":    "testProvideValue1",
 		"testProvideKey2":    "testProvideValue2",
 		"overrideProvideKey": "originalOverrideProvideValue",
-	}, *updProvides)
+	}, updProvides)
 	updProvides = handler.GetUpdateAugmentProvides()
 	assert.Equal(t, artifact.TypeInfoProvides{
 		"augmentProvideKey1": "augmentProvideValue1",
 		"augmentProvideKey2": "augmentProvideValue2",
 		"overrideProvideKey": "augmentOverrideProvideValue",
-	}, *updProvides)
+	}, updProvides)
 	updProvides, err = handler.GetUpdateProvides()
 	require.NoError(t, err)
 	assert.Equal(t, artifact.TypeInfoProvides{
@@ -293,7 +293,7 @@ func TestWriteModuleImage(t *testing.T) {
 		"augmentProvideKey1": "augmentProvideValue1",
 		"augmentProvideKey2": "augmentProvideValue2",
 		"overrideProvideKey": "augmentOverrideProvideValue",
-	}, *updProvides)
+	}, updProvides)
 
 	assert.Equal(t, map[string]interface{}{"metadata": "test"}, handler.GetUpdateOriginalMetaData())
 	assert.Equal(t, map[string]interface{}{"metadata": "augment"}, handler.GetUpdateAugmentMetaData())
@@ -312,3 +312,146 @@ func TestWriteModuleImage(t *testing.T) {
 	err = reader.ReadArtifact()
 	assert.Error(t, err)
 }
+
+func TestWriteRootfsArtifactDependsAndProvides(t *testing.T) {
+	tmpdir, err := ioutil.TempDir("", "mendertest")
+	require.NoError(t, err)
+	defer os.RemoveAll(tmpdir)
+	artfile := filepath.Join(tmpdir, "artifact.mender")
+
+	updateTestDir, _ := ioutil.TempDir("", "update")
+	defer os.RemoveAll(updateTestDir)
+
+	err = MakeFakeUpdateDir(updateTestDir,
+		[]TestDirEntry{
+			{
+				Path:    "update.ext4",
+				Content: []byte("my update"),
+				IsDir:   false,
+			},
+		})
+	assert.NoError(t, err)
+
+	os.Args = []string{
+		"mender-artifact", "write", "rootfs-image",
+		"-t", "mydevice",
+		"-o", artfile,
+		"-f", filepath.Join(updateTestDir, "update.ext4"),
+		"-n", "testName",
+		"-N", "testNameDepends1",
+		"-N", "testNameDepends2",
+		"-G", "testGroupDepends1",
+		"-G", "testGroupDepends2",
+		"-g", "testGroupProvide",
+		"-d", "testDependKey1:testDependValue1",
+		"-d", "testDependKey2:testDependValue2",
+		"-p", "testProvideKey1:testProvideValue1",
+		"-p", "testProvideKey2:testProvideValue2",
+	}
+	err = run()
+	assert.NoError(t, err)
+
+	artFd, err := os.Open(artfile)
+	assert.NoError(t, err)
+	reader := areader.NewReader(artFd)
+	err = reader.ReadArtifact()
+	assert.NoError(t, err)
+
+	// Verify name
+	assert.Equal(t, "testName", reader.GetArtifactName())
+	assert.Equal(t, []artifact.UpdateType{artifact.UpdateType{Type: "rootfs-image"}}, reader.GetUpdates())
+
+	// Verify Provides
+	provides := reader.GetArtifactProvides()
+	assert.NotNil(t, provides)
+	assert.Equal(t, "testName", provides.ArtifactName)
+	assert.Equal(t, "testGroupProvide", provides.ArtifactGroup)
+
+	// Verify Depends
+	depends := reader.GetArtifactDepends()
+	assert.NotNil(t, depends)
+	assert.Equal(t, []string{"testNameDepends1", "testNameDepends2"}, depends.ArtifactName)
+	assert.Equal(t, []string{"testGroupDepends1", "testGroupDepends2"}, depends.ArtifactGroup)
+
+	// Verify update
+	updates := reader.GetUpdates()
+	assert.Equal(t, 1, len(updates))
+	assert.Equal(t, "rootfs-image", updates[0].Type)
+	handlers := reader.GetHandlers()
+	assert.Equal(t, 1, len(handlers))
+	handler := handlers[0]
+	assert.Equal(t, "rootfs-image", handler.GetUpdateType())
+
+	// Type-Info Depends
+	updDepends, err := handler.GetUpdateDepends()
+	require.NoError(t, err)
+	assert.Equal(t, artifact.TypeInfoDepends{
+		"testDependKey1": "testDependValue1",
+		"testDependKey2": "testDependValue2",
+	}, updDepends)
+
+	// Type-Info Provides
+	updProvides, err := handler.GetUpdateProvides()
+	require.NoError(t, err)
+	assert.Equal(t, artifact.TypeInfoProvides{
+		// `rootfs_image_checksum` is always enabled
+		"rootfs_image_checksum": "bfb4567944c5730face9f3d54efc0c1ff3b5dd1338862b23b849ac87679e162f",
+		"testProvideKey1":       "testProvideValue1",
+		"testProvideKey2":       "testProvideValue2",
+	}, updProvides)
+
+	// Test the `--no-checksum-provide` flag
+	tart := filepath.Join(tmpdir, "noprovides.mender")
+
+	os.Args = []string{
+		"mender-artifact", "write", "rootfs-image",
+		"-t", "mydevice",
+		"-o", tart,
+		"-f", filepath.Join(updateTestDir, "update.ext4"),
+		"-n", "noprovides",
+		"--no-checksum-provide",
+	}
+	err = run()
+	assert.NoError(t, err)
+
+	artFd, err = os.Open(tart)
+	assert.NoError(t, err)
+	reader = areader.NewReader(artFd)
+	err = reader.ReadArtifact()
+	assert.NoError(t, err)
+
+	assert.Equal(t, "noprovides", reader.GetArtifactName())
+	assert.Equal(t, []artifact.UpdateType{artifact.UpdateType{Type: "rootfs-image"}}, reader.GetUpdates())
+
+	handlers = reader.GetHandlers()
+	assert.Equal(t, 1, len(handlers))
+	handler = handlers[0]
+	assert.Equal(t, "rootfs-image", handler.GetUpdateType())
+
+	updProvides, err = handler.GetUpdateProvides()
+	require.NoError(t, err)
+	// Nil represents empty provides unfortunately
+	assert.Nil(t, updProvides)
+
+}
+
+func TestWriteRootfsImageChecksum(t *testing.T) {
+
+	// Cannot find payload file (nonexisting)
+	err := writeRootfsImageChecksum("idonotexist", nil)
+	assert.Contains(t, err.Error(), "Failed to open the payload file")
+
+	// Checksum a dummy file
+	tf, err := ioutil.TempFile("", "TestWriteRootfsImageChecksum")
+	require.NoError(t, err)
+	_, err = tf.Write([]byte("foobar"))
+	require.NoError(t, err)
+	require.NoError(t, tf.Close())
+	typeInfo := artifact.TypeInfoV3{}
+	err = writeRootfsImageChecksum(tf.Name(), &typeInfo)
+	assert.NoError(t, err)
+	require.NotNil(t, typeInfo.ArtifactProvides)
+	_, ok := typeInfo.ArtifactProvides["rootfs_image_checksum"]
+	assert.True(t, ok)
+
+}
